@@ -11,7 +11,11 @@ use std::{
 
 use anyhow::{anyhow, Result};
 use rand::{thread_rng, RngCore};
-use sev::firmware::guest::{AttestationReport, Firmware};
+
+use sev::{
+    firmware::guest::{AttestationReport, Firmware},
+    parser::{ByteParser, Encoder},
+};
 
 // Read a bin-formatted attestation report.
 pub fn read_report(att_report_path: PathBuf) -> Result<AttestationReport, anyhow::Error> {
@@ -71,30 +75,30 @@ impl ReportArgs {
             ));
         }
 
-        if self.random && hyperv {
-            return Err(anyhow!(
-                "--random enabled yet Hyper-V guest detected (not allowed). Consult man page."
-            ));
-        }
-
         if self.platform && !hyperv {
-            return Err(anyhow!("--platform enabled yet Hyper-V guest not detected (not allowed). Consult man page."));
+            #[cfg(feature = "hyperv")]
+            let msg = "--platform enabled yet Hyper-V guest with SEV-SNP isolation not detected (not allowed). Consult man page.";
+            #[cfg(not(feature = "hyperv"))]
+            let msg =
+                "--platform requires a binary built with --features hyperv. Consult man page.";
+
+            return Err(anyhow!(msg));
         }
 
         Ok(())
     }
 }
 
-#[cfg(feature = "hyperv")]
 fn request_hardware_report(
-    _data: Option<[u8; 64]>,
+    data: Option<[u8; 64]>,
     vmpl: Option<u32>,
+    _platform: bool,
 ) -> Result<AttestationReport> {
-    hyperv::report::get(vmpl.unwrap_or(0))
-}
+    #[cfg(feature = "hyperv")]
+    if _platform {
+        return hyperv::report::get(vmpl.unwrap_or(0));
+    }
 
-#[cfg(not(feature = "hyperv"))]
-fn request_hardware_report(data: Option<[u8; 64]>, vmpl: Option<u32>) -> Result<AttestationReport> {
     let mut fw = Firmware::open().context("unable to open /dev/sev-guest")?;
     Ok(AttestationReport::from_bytes(
         fw.get_report(None, data, vmpl)
@@ -123,7 +127,7 @@ pub fn get_report(args: ReportArgs, hv: bool) -> Result<()> {
         Some(bytes)
     };
 
-    let report = request_hardware_report(data, args.vmpl)?;
+    let report = request_hardware_report(data, args.vmpl, args.platform)?;
 
     /*
      * Serialize and write attestation report.
@@ -134,22 +138,29 @@ pub fn get_report(args: ReportArgs, hv: bool) -> Result<()> {
         .write(true)
         .open(&args.att_report_path)?;
 
-    report.write_bytes(&mut file)?;
+    report.encode(&mut file, ())?;
 
     /*
      * Write reports report data (only for --random or --platform).
      */
     if args.random {
-        reqdata_write(args.request_file, &report).context("unable to write random request data")?;
+        if let Some(data) = data {
+            reqdata_write(args.request_file, &data)
+                .context("unable to write random request data to specified file")?;
+        } else {
+            return Err(anyhow!("unable to write empty buffer to specified file."));
+        }
     } else if args.platform {
-        reqdata_write(args.request_file, &report)
+        // Because random data cannot be provided for platform, we will pull the
+        // data provided by the vTPM from the report.
+        reqdata_write(args.request_file, &report.report_data)
             .context("unable to write platform request data")?;
     }
 
     Ok(())
 }
 
-fn reqdata_write(name: PathBuf, report: &AttestationReport) -> Result<()> {
+fn reqdata_write(name: PathBuf, report_data: &[u8]) -> Result<()> {
     let mut file = OpenOptions::new()
         .create(true)
         .truncate(true)
@@ -157,21 +168,6 @@ fn reqdata_write(name: PathBuf, report: &AttestationReport) -> Result<()> {
         .open(name)
         .context("unable to create or write to request data file")?;
 
-    write_hex(&mut file, report.report_data.as_slice())
+    file.write_all(report_data)
         .context("unable to write report data to REQUEST_FILE")
 }
-
-pub fn write_hex(file: &mut File, data: &[u8]) -> Result<()> {
-    let mut line_counter = 0;
-    for val in data {
-        // Make it blocks for easier read
-        if line_counter.eq(&16) {
-            writeln!(file)?;
-            line_counter = 0;
-        }
-
-        write!(file, "{:02x}", val)?;
-        line_counter += 1;
-    }
-    Ok(())
-}
