@@ -14,6 +14,9 @@ use std::{
 
 use openssl::{ecdsa::EcdsaSig, sha::Sha384};
 use sev::certs::snp::Chain;
+use sev::parser::ByteParser;
+
+use hex::FromHex;
 
 #[derive(Subcommand)]
 pub enum VerifyCmd {
@@ -38,7 +41,7 @@ pub fn find_cert_in_dir(dir: &Path, cert: &str) -> Result<PathBuf, anyhow::Error
     } else if dir.join(format!("{cert}.der")).exists() {
         Ok(dir.join(format!("{cert}.der")))
     } else {
-        return Err(anyhow::anyhow!("{cert} certificate not found in directory"));
+        Err(anyhow::anyhow!("{cert} certificate not found in directory"))
     }
 }
 
@@ -167,7 +170,7 @@ mod attestation {
 
     // OID extensions for the VCEK, will be used to verify attestation report
     impl SnpOid {
-        fn oid(&self) -> Oid {
+        fn oid(&self) -> Oid<'static> {
             match self {
                 SnpOid::BootLoader => oid!(1.3.6 .1 .4 .1 .3704 .1 .3 .1),
                 SnpOid::Tee => oid!(1.3.6 .1 .4 .1 .3704 .1 .3 .2),
@@ -200,6 +203,18 @@ mod attestation {
         /// Run the Signature Verification Exclusively.
         #[arg(short, long, conflicts_with = "tcb")]
         pub signature: bool,
+
+        /// Optional measurement string (hex, 48 bytes / 96 chars)
+        #[arg(short, long, value_name = "measurement")]
+        pub measurement: Option<String>,
+
+        /// Optional host_data string (hex, 32 bytes / 64 chars)
+        #[arg(short = 'd', long, value_name = "host_data")]
+        pub host_data: Option<String>,
+
+        /// Optional report_data string (hex, 64 bytes / 128 chars)
+        #[arg(short, long, value_name = "report_data")]
+        pub report_data: Option<String>,
     }
 
     fn verify_attestation_signature(
@@ -216,8 +231,7 @@ mod attestation {
         // Get the attestation report signature
         let ar_signature = EcdsaSig::try_from(&att_report.signature)
             .context("Failed to get ECDSA Signature from attestation report.")?;
-        let mut report_bytes = Vec::new();
-        att_report.write_bytes(&mut report_bytes)?;
+        let report_bytes = att_report.to_bytes()?;
         let signed_bytes = &report_bytes[0x0..0x2A0];
 
         let mut hasher: Sha384 = Sha384::new();
@@ -369,7 +383,7 @@ mod attestation {
         // Compare HWID information only on VCEK
         if common_name == CertType::VCEK {
             if let Some(cert_hwid) = extensions.get(&SnpOid::HwId.oid()) {
-                if !check_cert_bytes(cert_hwid, &*att_report.chip_id) {
+                if !check_cert_bytes(cert_hwid, &att_report.chip_id) {
                     return Err(anyhow::anyhow!(
                         "Report TCB ID and Certificate ID mismatch encountered."
                     ));
@@ -418,7 +432,7 @@ mod attestation {
                 .context("Could not open attestation report")?
         };
 
-        let proc_model = if let Some(proc_model) = args.processor_model {
+        let proc_model = if let Some(proc_model) = args.processor_model.clone() {
             proc_model
         } else {
             let att_report = report::read_report(args.att_report_path.clone())
@@ -447,6 +461,89 @@ mod attestation {
             verify_attestation_signature(vek, att_report, quiet)?;
         }
 
+        // Verify optional fields if provided
+        if args.measurement.is_some() || args.host_data.is_some() || args.report_data.is_some() {
+            verify_report_fields(&args, &att_report, quiet)?;
+        }
+
+        Ok(())
+    }
+
+    fn decode_hex_or_decimal(input: &str) -> Result<Vec<u8>> {
+        // Look for "0x" at beginning. If it exists, treat as a hex.
+        if let Some(hex_str) = input.strip_prefix("0x") {
+            Ok(Vec::from_hex(hex_str)?)
+        } else {
+            Ok(input.as_bytes().to_vec())
+        }
+    }
+
+    fn verify_field(
+        field_name: &str,
+        expected: &[u8],
+        provided: &str,
+        expected_len: usize,
+        quiet: bool,
+    ) -> Result<()> {
+        let actual = decode_hex_or_decimal(provided)?;
+
+        if actual.len() != expected_len {
+            return Err(anyhow::anyhow!(
+                "Expected {} characters for {}, got {}",
+                expected_len,
+                field_name,
+                actual.len()
+            ));
+        }
+
+        if expected != actual.as_slice() {
+            return Err(anyhow::anyhow!(
+                "{} did not match:\n expected {}\n got {}",
+                field_name,
+                hex::encode(expected),
+                hex::encode(actual)
+            ));
+        }
+        if !quiet {
+            println!("{} verified successfully.", field_name);
+        }
+
+        Ok(())
+    }
+
+    fn verify_report_fields(
+        args: &Args,
+        att_report: &AttestationReport,
+        quiet: bool,
+    ) -> Result<()> {
+        if let Some(measure) = &args.measurement {
+            verify_field(
+                "Measurement",
+                att_report.measurement.as_slice(),
+                measure,
+                48,
+                quiet,
+            )?;
+        }
+        if let Some(host) = &args.host_data {
+            verify_field(
+                "Host Data",
+                att_report.host_data.as_slice(),
+                host,
+                32,
+                quiet,
+            )?;
+        }
+        if let Some(report) = &args.report_data {
+            verify_field(
+                "Report Data",
+                att_report.report_data.as_slice(),
+                report,
+                64,
+                quiet,
+            )?;
+        }
+
         Ok(())
     }
 
